/**
 * Copyright (c) 2007 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/


/**
 * @file serial.h
 * @brief Contains declarations for serial input and output
 *
 * Contains definitions for serial input and output.
 *
 * @author Colony Project, CMU Robotics Club
 * Based on Tom Lauwer's Firefly Library
 *
 **/

/*
  serial.h - Contains definitions and function prototypes for the RS232 serial port
  author(s): pkv
  
  Directions:
  Call the initialization function for the serial port you wish to use.  Then, use
  either the provided functions or the stdio functions (fprintf, etc) to read and
  write characters to the serial ports.
  
  UART Mapping:
  usb_*() -> UART0
  xbee_*() -> UART1
  
  Options: (Add the following defines to your code to configure this library)
  #define USB_BAUD { 115200 | 9600 } <= pick ONE value from in here
  #define XBEE_BAUD { 115200 | 9600 } <= pick ONE value from in here
  #define USE_STDIO
  
  Note: If you enable USE_STDIO, the first init function that is called will 
  automatically be linked to stdin, stdout, and stderr.  To use the baud rate 
  commands, add something like the following to your code:
  
  #define FOO_BAUD 9600
  
  **UNLESS YOU KNOW WHAT YOU ARE DOING, PLEASE DO NOT CHANGE THIS FILE**
  Many, many other people use this file in their code.  If you change it, you will
  probably break all of their nice code.  You should not need to change anything in
  here, except to accomodate new hardware.
*/

#ifndef _SERIAL_H
#define _SERIAL_H

#include <inttypes.h>
#include <avr/pgmspace.h>

/**
 * @defgroup usb USB Input / Output
 * @brief Functions for USB input / output
 *
 * Low level functions for USB input and output.
 *
 * @{
 **/

// if no baud rate is defined for usb, default is set here
#ifndef USB_BAUD
/** @brief the USB baud rate **/
#define USB_BAUD 115200
#endif

/** @brief Initialize the USB **/
void usb_init(void);
/** @brief Print a character to USB **/
int usb_putc(char c);
/** @brief Read a character from USB **/
int usb_getc(void);
/** @brief Read a character from USB without blocking **/
int usb_getc_nb(char *c);
/** @brief Print a string to USB **/
int usb_puts(char *s);
/** @brief Print a string from program space to USB **/
void usb_puts_P (PGM_P s);
/** @brief Print an integer to USB **/
int usb_puti(int value);
/** @brief Determine a hexadecimal digit **/
uint8_t hex_digit (uint8_t value);
/** @brief Print a fixed width hexadecimal representation to USB **/
void usb_puth16 (uint16_t value);
/** @brief Print a fixed width hexadecimal representation to USB **/
void usb_puth8(uint8_t value);
/** @brief Alias for usb_puth16 **/
static inline void usb_puth (uint16_t value) { usb_puth16 (value); };


/** @} **/ //end addtogroup

/**
 * @defgroup xbee XBee Input / Output
 * @brief Functions for XBee input / output
 *
 * Low level functions for XBee input and output.
 *
 * @{
 **/

// if no baud rate is defined for usb, default is set here

// if no baud rate is defined for xbee, default is set here
#ifndef XBEE_BAUD
/** @brief the XBee baud rate **/
#define XBEE_BAUD 9600
#endif

/** @brief Initialize the XBee **/
void xbee_init(void);
/** @brief Print a character to the XBee **/
int xbee_putc(char c);
/** @brief Read a character from the XBee **/
int xbee_getc(void);
/** @brief Read a character from the XBee without blocking **/
int xbee_getc_nb(char *c);



/** @} **/ //end addtogroup

#endif

